#include "SoundClasses.h"
#include <DSound.h>


//-Static Variables---------------------------------------------------------------------------
unsigned int			Listener::mDevicesNum	= 0;
tAudioDeviceInfo		Listener::mDevices[MAXDEVICES];
LPDIRECTSOUND			Listener::mlpDS			= NULL;
LPDIRECTSOUNDBUFFER		Listener::mPrimary		= NULL;
//-------------------------------------------------------------------
Listener::Listener()
{		
	EnumDevices();	
}
//-------------------------------------------------------------------
Listener::~Listener()
{		
	DeallocDevicesList();
	if (mlpDS	!= NULL) mlpDS->Release();
	if (mPrimary!= NULL) mPrimary->Release();
}
//-------------------------------------------------------------------
bool Listener::Init(HWND hWindow, unsigned int Device, int nChannels, int samplesPerSec, int bitsPerSample)
{
	if (mlpDS != NULL) return true;	
	if (FAILED(DirectSoundCreate(mDevices[Device].mGUID, &mlpDS, NULL)))return false;
	if (nChannels == 0)	{
		if (FAILED(mlpDS->SetCooperativeLevel(hWindow, DSSCL_NORMAL)))	return false;
	} else	{
		if (FAILED(mlpDS->SetCooperativeLevel(hWindow, DSSCL_PRIORITY)))		return false;
		if (FAILED(CreatePrimaryBuffer(nChannels, samplesPerSec, bitsPerSample)))return false;
	}
	return true;
}
//-------------------------------------------------------------------
bool Listener::EnumDevices()
{
	return (!FAILED(DirectSoundEnumerate((LPDSENUMCALLBACK)EnumDevicesProc, NULL)));
}
//-------------------------------------------------------------------
void Listener::DeallocDevicesList()
{
	for (unsigned int i = 0; i < mDevicesNum; i++)	{
		if (mDevices[i].mDescr	!= NULL)	delete  [] mDevices[i].mDescr;
		if (mDevices[i].mDrvName != NULL)	delete  [] mDevices[i].mDrvName;
		mDevices[i].mDescr	= NULL;
		mDevices[i].mDrvName= NULL;
	}
}
//-------------------------------------------------------------------
bool CALLBACK Listener::EnumDevicesProc(LPGUID lpGUID, char *lpszDesc, char *lpszDrvName, void * lpContext)
{	
	mDevices[mDevicesNum].mGUID	= lpGUID;
	mDevices[mDevicesNum].mDescr	=  (strlen(lpszDesc) > 0)	?	strdup(lpszDesc)	: NULL;
	mDevices[mDevicesNum].mDrvName	=  (strlen(lpszDrvName) > 0)?	strdup(lpszDrvName) : NULL;	
	mDevicesNum++;
	return (mDevicesNum < MAXDEVICES);	
}
//-------------------------------------------------------------------
bool Listener::SetSpeakerMode(unsigned int mode)
{	/*DSSPEAKER_HEADPHONE 
	The speakers are headphones. 
	DSSPEAKER_MONO 
	The speakers are monaural. 
	DSSPEAKER_QUAD 
	The speakers are quadraphonic. 
	DSSPEAKER_STEREO 
	The speakers are stereo (default value). 
	DSSPEAKER_SURROUND 
	The speakers are surround sound. 

	DSSPEAKER_STEREO may be combined with one of the following values: 
	DSSPEAKER_GEOMETRY_WIDE 
	The speakers are directed over an arc of 20 degrees. 
	DSSPEAKER_GEOMETRY_NARROW 
	The speakers are directed over an arc of 10 degrees. 
	DSSPEAKER_GEOMETRY_MIN 
	The speakers are directed over an arc of 5 degrees. 
	DSSPEAKER_GEOMETRY_MAX 
	The speakers are directed over an arc of 180 degrees. */
	return (mlpDS->SetSpeakerConfig(mode) == DS_OK);
}
//-------------------------------------------------------------------
bool Listener::CreatePrimaryBuffer(int nChannels, int samplesPerSec, int bitsPerSample, unsigned int BufferProperties)
{
	DSCAPS DSCaps;
	DSCaps.dwSize	= sizeof(DSCAPS);
	if (FAILED(mlpDS->GetCaps(&DSCaps))) return false;	
	
	if ((nChannels == 1)	  && (!(DSCaps.dwFlags & DSCAPS_PRIMARYMONO)))	return false;
	if ((nChannels == 2)	  && (!(DSCaps.dwFlags & DSCAPS_PRIMARYSTEREO)))return false;
	if ((bitsPerSample == 8)  && (!(DSCaps.dwFlags & DSCAPS_PRIMARY8BIT)))	return false;
	if ((bitsPerSample == 16) && (!(DSCaps.dwFlags & DSCAPS_PRIMARY16BIT)))	return false;

	WAVEFORMATEX  WaveFormat;
	memset(&WaveFormat,0,sizeof(WAVEFORMATEX));
	WaveFormat.wFormatTag     = WAVE_FORMAT_PCM;
	WaveFormat.nChannels      = nChannels;
	WaveFormat.nSamplesPerSec = samplesPerSec;
	WaveFormat.nBlockAlign    = (nChannels * bitsPerSample) >> 3;
	WaveFormat.nAvgBytesPerSec= samplesPerSec * WaveFormat.nBlockAlign;
	WaveFormat.wBitsPerSample = bitsPerSample;

	DSBUFFERDESC  DSBDesc;
	memset(&DSBDesc,0, sizeof(DSBUFFERDESC));
	DSBDesc.dwSize			= sizeof(DSBUFFERDESC);	
	DSBDesc.dwBufferBytes	= 0;
	DSBDesc.lpwfxFormat		= NULL;
	DSBDesc.dwFlags			= DSBCAPS_PRIMARYBUFFER | BufferProperties;	
			
	if (FAILED(mlpDS->CreateSoundBuffer(&DSBDesc,&mPrimary,NULL)))	return false;
	if (FAILED(mPrimary->SetFormat(&WaveFormat))) 					return false;
	
	return true;
}
//-------------------------------------------------------------------
