#include <DSound.h>
#include <stdio.h>
#include "SoundClasses.h"

//-------------------------------------------------------------------
SimpleSound::SimpleSound()
{
	mBuffer = NULL;
}
//-------------------------------------------------------------------
SimpleSound::SimpleSound(char *FileName)
{
	LoadWave(FileName);
}
//-------------------------------------------------------------------
SimpleSound::SimpleSound(SimpleSound &sound)
{
	mBuffer = NULL;
	if (Listener::GetDSObject() != NULL) 
		Listener::GetDSObject()->DuplicateSoundBuffer(sound.mBuffer, &mBuffer);
}
//-------------------------------------------------------------------
SimpleSound::~SimpleSound()
{
	if (mBuffer != NULL) mBuffer->Release();
}
//-------------------------------------------------------------------
bool SimpleSound::LoadWave(char *FileName)
{
	tWaveHeader		waveHdr;

	if (Listener::GetDSObject() == NULL) return false;
	
	FILE* pFile = fopen(FileName, "rb");
	if(!pFile) return false;

	if (fread(&waveHdr, sizeof(tWaveHeader), 1, pFile) != 1) 	{
		fclose(pFile);
		return false;
	}
	
	if (!AllocBuffer(waveHdr.wChnls,
					 waveHdr.dwSRate,
					 waveHdr.BitsPerSample,
					 waveHdr.dwDSize,
					 DSBCAPS_CTRLFREQUENCY | DSBCAPS_CTRLVOLUME)) return false;
	
	if (mBuffer != NULL) {
		void			*Data;
		unsigned long	DataSize;

		if (FAILED(mBuffer->Lock( 0, 0, &Data, &DataSize, NULL, NULL, DSBLOCK_ENTIREBUFFER))) return false;
		fread(Data, DataSize, 1, pFile);
		if (FAILED(mBuffer->Unlock(Data, DataSize, NULL, 0))) return false;
	}
	
	fclose(pFile);
	
	if (mBuffer != NULL)	return false;
	
	return true;
}
//-------------------------------------------------------------------
void SimpleSound::Play(bool Looping)
{	
	if (mBuffer != NULL)	mBuffer->Play(NULL, NULL, (Looping) ? DSBPLAY_LOOPING : NULL);
}
//-------------------------------------------------------------------
void SimpleSound::Stop()
{	
	if (mBuffer != NULL)	mBuffer->Stop();
}
//-------------------------------------------------------------------
unsigned long SimpleSound::GetFrequency()
{
	unsigned long Frequency;
	if (mBuffer != NULL)	{
		if (FAILED(mBuffer->GetFrequency(&Frequency))) return 0;
		return Frequency;
	}
	return 0;
}
//-------------------------------------------------------------------
bool SimpleSound::SetFrequency(unsigned long Frequency)
{
	if (mBuffer != NULL)	return (!FAILED(mBuffer->SetFrequency(Frequency)));
	
	return false;
}
//-------------------------------------------------------------------
long SimpleSound::GetVolume()
{
	long Volume;
	if (mBuffer != NULL)	{
		if (FAILED(mBuffer->GetVolume(&Volume))) return 0;
		return Volume;
	}
	return 0;
}
//-------------------------------------------------------------------
bool SimpleSound::SetVolume(long Volume)
{
	if (mBuffer != NULL)	return (!FAILED(mBuffer->SetVolume(Volume)));
	
	return false;
}
//-------------------------------------------------------------------
bool SimpleSound::AllocBuffer(int nChannels, int samplesPerSec, int bitsPerSample, unsigned int size, unsigned int BufferProperties)
{
	WAVEFORMATEX  WaveFormat;
	memset(&WaveFormat,0,sizeof(WAVEFORMATEX));
	WaveFormat.wFormatTag     = WAVE_FORMAT_PCM;
	WaveFormat.nChannels      = nChannels;
	WaveFormat.nSamplesPerSec = samplesPerSec;
	WaveFormat.nBlockAlign    = (nChannels * bitsPerSample) >> 3;
	WaveFormat.nAvgBytesPerSec= samplesPerSec * WaveFormat.nBlockAlign;
	WaveFormat.wBitsPerSample = bitsPerSample;
	
	DSBUFFERDESC  DSBDesc;
	memset(&DSBDesc,0, sizeof(DSBUFFERDESC));
	DSBDesc.dwSize			= sizeof(DSBUFFERDESC);
	DSBDesc.dwFlags		   |= BufferProperties;
	DSBDesc.dwBufferBytes	= size;
	DSBDesc.lpwfxFormat		= &WaveFormat;

	if (FAILED(Listener::GetDSObject()->CreateSoundBuffer(&DSBDesc, &mBuffer, NULL))) return false;
	return true;
}
//-------------------------------------------------------------------

